@@ -94,9 +94,6 @@ static void      panel_application_drag_leave         (GtkWidget              *w
                                                        GdkDragContext         *context,
                                                        guint                   drag_time,
                                                        PanelApplication       *application);
-static void      panel_application_save_window        (PanelApplication       *application,
-                                                       PanelWindow            *window,
-                                                       PanelSaveTypes          save_types);
 
 
 
@@ -317,6 +314,7 @@ panel_application_load_real (PanelApplication *application)
   GValue        val = { 0, };
   GPtrArray    *panels;
   gint          panel_id;
+  gboolean      save_changed_ids = FALSE;
 
   panel_return_if_fail (PANEL_IS_APPLICATION (application));
   panel_return_if_fail (XFCONF_IS_CHANNEL (application->xfconf));
@@ -402,6 +400,9 @@ panel_application_load_real (PanelApplication *application)
                   /* show warnings */
                   g_message ("Plugin \"%s-%d\" was not found and has been "
                              "removed from the configuration", name, unique_id);
+
+                  /* save configuration change after loading */
+                  save_changed_ids = TRUE;
                 }
 
               g_free (name);
@@ -417,6 +418,9 @@ panel_application_load_real (PanelApplication *application)
   /* create empty window if everything else failed */
   if (G_UNLIKELY (application->windows == NULL))
     panel_application_new_window (application, NULL, -1, TRUE);
+
+  if (save_changed_ids)
+    panel_application_save (application, SAVE_PLUGIN_IDS);
 }
 
 
@@ -512,8 +516,8 @@ panel_application_plugin_move_drag_end (GtkWidget        *item,
   g_signal_handlers_disconnect_by_func (G_OBJECT (item),
       G_CALLBACK (panel_application_plugin_move_drag_data_get), application);
 
-  /* make the window sensitive again */
-  panel_application_windows_sensitive (application, TRUE);
+  /* unblock autohide */
+  panel_application_windows_blocked (application, FALSE);
 }
 
 
@@ -531,8 +535,8 @@ panel_application_plugin_move (GtkWidget        *item,
   panel_return_if_fail (XFCE_IS_PANEL_PLUGIN_PROVIDER (item));
   panel_return_if_fail (PANEL_IS_APPLICATION (application));
 
-  /* make the window insensitive */
-  panel_application_windows_sensitive (application, FALSE);
+  /* block autohide */
+  panel_application_windows_blocked (application, TRUE);
 
   /* create drag context */
   target_list = gtk_target_list_new (drag_targets, G_N_ELEMENTS (drag_targets));
@@ -685,6 +689,9 @@ panel_application_plugin_provider_signal (XfcePanelPluginProvider       *provide
           /* remove the plugin configuration */
           panel_application_plugin_delete_config (application, name, unique_id);
           g_free (name);
+
+          /* save new ids */
+          panel_application_save_window (application, window, SAVE_PLUGIN_IDS);
         }
       break;
 
@@ -785,25 +792,6 @@ panel_application_plugin_insert (PanelApplication  *application,
 
 
 
-static void
-panel_application_windows_block_autohide (PanelApplication *application,
-                                          gboolean          blocked)
-{
-  GSList *li;
-
-  panel_return_if_fail (PANEL_IS_APPLICATION (application));
-
-  for (li = application->windows; li != NULL; li = li->next)
-    {
-      if (blocked)
-        panel_window_freeze_autohide (PANEL_WINDOW (li->data));
-      else
-        panel_window_thaw_autohide (PANEL_WINDOW (li->data));
-    }
-}
-
-
-
 static void
 panel_application_dialog_destroyed (GtkWindow        *dialog,
                                     PanelApplication *application)
@@ -817,7 +805,7 @@ panel_application_dialog_destroyed (GtkWindow        *dialog,
 
   /* unblock autohide if there are no open windows anymore */
   if (application->dialogs == NULL)
-    panel_application_windows_block_autohide (application, FALSE);
+    panel_application_windows_blocked (application, FALSE);
 }
 
 
@@ -841,6 +829,7 @@ panel_application_drag_data_received (PanelWindow      *window,
   gchar            **uris;
   guint              i;
   gboolean           found;
+  gint               n_items;
 
   panel_return_if_fail (PANEL_IS_WINDOW (window));
   panel_return_if_fail (GDK_IS_DRAG_CONTEXT (context));
@@ -943,9 +932,20 @@ panel_application_drag_data_received (PanelWindow      *window,
               uris = gtk_selection_data_get_uris (selection_data);
               if (G_LIKELY (uris != NULL))
                 {
-                  /* create a new item with a unique id */
-                  succeed = panel_application_plugin_insert (application, window, LAUNCHER_PLUGIN_NAME,
-                                                             -1, uris, application->drop_index);
+                  n_items = g_strv_length (uris);
+                  if (xfce_dialog_confirm (NULL, GTK_STOCK_ADD, _("Create _Launcher"),
+                                           _("This will create a new launcher plugin on the panel and inserts "
+                                             "the dropped files as menu items."),
+                                           ngettext ("Create new launcher from %d desktop file",
+                                                     "Create new launcher from %d desktop files",
+                                                     n_items),
+                                           n_items))
+                    {
+                      /* create a new item with a unique id */
+                      succeed = panel_application_plugin_insert (application, window, LAUNCHER_PLUGIN_NAME,
+                                                                 -1, uris, application->drop_index);
+                    }
+
                   g_strfreev (uris);
                 }
 
@@ -1118,89 +1118,6 @@ panel_application_window_id_exists (PanelApplication *application,
 
 
 
-static void
-panel_application_save_window (PanelApplication *application,
-                               PanelWindow      *window,
-                               PanelSaveTypes    save_types)
-{
-  GList                   *children, *lp;
-  GtkWidget               *itembar;
-  XfcePanelPluginProvider *provider;
-  gchar                    buf[50];
-  XfconfChannel           *channel = application->xfconf;
-  GPtrArray               *array = NULL;
-  GValue                  *value;
-  gint                     plugin_id;
-  gint                     panel_id;
-
-  /* skip this window if it is locked */
-  if (panel_window_get_locked (window)
-      || !PANEL_HAS_FLAG (save_types, SAVE_PLUGIN_IDS | SAVE_PLUGIN_PROVIDERS))
-    return;
-
-  panel_id = panel_window_get_id (window);
-  panel_debug (PANEL_DEBUG_APPLICATION,
-               "saving /panels/panel-%d: ids=%s, providers=%s",
-               panel_id,
-               PANEL_DEBUG_BOOL (PANEL_HAS_FLAG (save_types, SAVE_PLUGIN_IDS)),
-               PANEL_DEBUG_BOOL (PANEL_HAS_FLAG (save_types, SAVE_PLUGIN_PROVIDERS)));
-
-  /* get the itembar children */
-  itembar = gtk_bin_get_child (GTK_BIN (window));
-  children = gtk_container_get_children (GTK_CONTAINER (itembar));
-
-  /* only cleanup and continue if there are no children */
-  if (PANEL_HAS_FLAG (save_types, SAVE_PLUGIN_IDS))
-    {
-      if (G_UNLIKELY (children == NULL))
-        {
-          g_snprintf (buf, sizeof (buf), "/panels/panel-%d/plugin-ids", panel_id);
-          if (xfconf_channel_has_property (channel, buf))
-            xfconf_channel_reset_property (channel, buf, FALSE);
-          return;
-        }
-
-      array = g_ptr_array_new ();
-    }
-
-  /* walk all the plugin children */
-  for (lp = children; lp != NULL; lp = lp->next)
-    {
-      provider = XFCE_PANEL_PLUGIN_PROVIDER (lp->data);
-
-      if (array != NULL)
-        {
-          plugin_id = xfce_panel_plugin_provider_get_unique_id (provider);
-
-          /* add plugin id to the array */
-          value = g_new0 (GValue, 1);
-          g_value_init (value, G_TYPE_INT);
-          g_value_set_int (value, plugin_id);
-          g_ptr_array_add (array, value);
-
-          /* make sure the plugin type-name is store in the plugin item */
-          g_snprintf (buf, sizeof (buf), "/plugins/plugin-%d", plugin_id);
-          xfconf_channel_set_string (channel, buf, xfce_panel_plugin_provider_get_name (provider));
-        }
-
-      /* ask the plugin to save */
-      if (PANEL_HAS_FLAG (save_types, SAVE_PLUGIN_PROVIDERS))
-        xfce_panel_plugin_provider_save (provider);
-    }
-
-  if (array != NULL)
-    {
-      /* store the plugin ids for this panel */
-      g_snprintf (buf, sizeof (buf), "/panels/panel-%d/plugin-ids", panel_id);
-      xfconf_channel_set_arrayv (channel, buf, array);
-      xfconf_array_free (array);
-    }
-
-  g_list_free (children);
-}
-
-
-
 PanelApplication *
 panel_application_get (void)
 {
@@ -1317,6 +1234,92 @@ panel_application_save (PanelApplication *application,
 
 
 
+void
+panel_application_save_window (PanelApplication *application,
+                               PanelWindow      *window,
+                               PanelSaveTypes    save_types)
+{
+  GList                   *children, *lp;
+  GtkWidget               *itembar;
+  XfcePanelPluginProvider *provider;
+  gchar                    buf[50];
+  XfconfChannel           *channel = application->xfconf;
+  GPtrArray               *array = NULL;
+  GValue                  *value;
+  gint                     plugin_id;
+  gint                     panel_id;
+
+  panel_return_if_fail (PANEL_IS_APPLICATION (application));
+  panel_return_if_fail (PANEL_IS_WINDOW (window));
+
+  /* skip this window if it is locked */
+  if (panel_window_get_locked (window)
+      || !PANEL_HAS_FLAG (save_types, SAVE_PLUGIN_IDS | SAVE_PLUGIN_PROVIDERS))
+    return;
+
+  panel_id = panel_window_get_id (window);
+  panel_debug (PANEL_DEBUG_APPLICATION,
+               "saving /panels/panel-%d: ids=%s, providers=%s",
+               panel_id,
+               PANEL_DEBUG_BOOL (PANEL_HAS_FLAG (save_types, SAVE_PLUGIN_IDS)),
+               PANEL_DEBUG_BOOL (PANEL_HAS_FLAG (save_types, SAVE_PLUGIN_PROVIDERS)));
+
+  /* get the itembar children */
+  itembar = gtk_bin_get_child (GTK_BIN (window));
+  children = gtk_container_get_children (GTK_CONTAINER (itembar));
+
+  /* only cleanup and continue if there are no children */
+  if (PANEL_HAS_FLAG (save_types, SAVE_PLUGIN_IDS))
+    {
+      if (G_UNLIKELY (children == NULL))
+        {
+          g_snprintf (buf, sizeof (buf), "/panels/panel-%d/plugin-ids", panel_id);
+          if (xfconf_channel_has_property (channel, buf))
+            xfconf_channel_reset_property (channel, buf, FALSE);
+          return;
+        }
+
+      array = g_ptr_array_new ();
+    }
+
+  /* walk all the plugin children */
+  for (lp = children; lp != NULL; lp = lp->next)
+    {
+      provider = XFCE_PANEL_PLUGIN_PROVIDER (lp->data);
+
+      if (array != NULL)
+        {
+          plugin_id = xfce_panel_plugin_provider_get_unique_id (provider);
+
+          /* add plugin id to the array */
+          value = g_new0 (GValue, 1);
+          g_value_init (value, G_TYPE_INT);
+          g_value_set_int (value, plugin_id);
+          g_ptr_array_add (array, value);
+
+          /* make sure the plugin type-name is store in the plugin item */
+          g_snprintf (buf, sizeof (buf), "/plugins/plugin-%d", plugin_id);
+          xfconf_channel_set_string (channel, buf, xfce_panel_plugin_provider_get_name (provider));
+        }
+
+      /* ask the plugin to save */
+      if (PANEL_HAS_FLAG (save_types, SAVE_PLUGIN_PROVIDERS))
+        xfce_panel_plugin_provider_save (provider);
+    }
+
+  if (array != NULL)
+    {
+      /* store the plugin ids for this panel */
+      g_snprintf (buf, sizeof (buf), "/panels/panel-%d/plugin-ids", panel_id);
+      xfconf_channel_set_arrayv (channel, buf, array);
+      xfconf_array_free (array);
+    }
+
+  g_list_free (children);
+}
+
+
+
 void
 panel_application_take_dialog (PanelApplication *application,
                                GtkWindow        *dialog)
@@ -1326,7 +1329,7 @@ panel_application_take_dialog (PanelApplication *application,
 
   /* block autohide if this will be the first dialog */
   if (application->dialogs == NULL)
-    panel_application_windows_block_autohide (application, TRUE);
+    panel_application_windows_blocked (application, TRUE);
 
   /* monitor window destruction */
   g_signal_connect (G_OBJECT (dialog), "destroy",
@@ -1357,15 +1360,16 @@ panel_application_destroy_dialogs (PanelApplication *application)
 
 void
 panel_application_add_new_item (PanelApplication  *application,
+                                PanelWindow       *window,
                                 const gchar       *plugin_name,
                                 gchar            **arguments)
 {
-  PanelWindow *window = NULL;
-  gint         panel_id;
+  gint panel_id;
 
   panel_return_if_fail (PANEL_IS_APPLICATION (application));
   panel_return_if_fail (plugin_name != NULL);
   panel_return_if_fail (application->windows != NULL);
+  panel_return_if_fail (window == NULL || PANEL_IS_WINDOW (window));
 
   /* leave if the config is locked */
   if (panel_application_get_locked (application))
@@ -1373,27 +1377,30 @@ panel_application_add_new_item (PanelApplication  *application,
 
   if (panel_module_factory_has_module (application->factory, plugin_name))
     {
-      /* find a suitable panel if there are 2 or more panel */
-      if (LIST_HAS_TWO_OR_MORE_ENTRIES (application->windows))
+      if (window == NULL)
         {
-          /* ask the user to select a panel */
-          panel_id = panel_dialogs_choose_panel (application);
-          if (panel_id == -1)
+          /* find a suitable panel if there are 2 or more panel */
+          if (LIST_HAS_TWO_OR_MORE_ENTRIES (application->windows))
             {
-              /* cancel was clicked */
-              return;
+              /* ask the user to select a panel */
+              panel_id = panel_dialogs_choose_panel (application);
+              if (panel_id == -1)
+                {
+                  /* cancel was clicked */
+                  return;
+                }
+              else
+                {
+                  /* get panel from the id */
+                  window = panel_application_get_window (application, panel_id);
+                }
             }
           else
             {
-              /* get panel from the id */
-              window = panel_application_get_window (application, panel_id);
+              /* get the first (and only) window */
+              window = g_slist_nth_data (application->windows, 0);
             }
         }
-      else
-        {
-          /* get the first (and only) window */
-          window = g_slist_nth_data (application->windows, 0);
-        }
 
       if (window != NULL && !panel_window_get_locked (window))
         {
@@ -1599,8 +1606,8 @@ panel_application_window_select (PanelApplication *application,
 
 
 void
-panel_application_windows_sensitive (PanelApplication *application,
-                                     gboolean          sensitive)
+panel_application_windows_blocked (PanelApplication *application,
+                                   gboolean          blocked)
 {
   GSList *li;
 
@@ -1610,10 +1617,10 @@ panel_application_windows_sensitive (PanelApplication *application,
   for (li = application->windows; li != NULL; li = li->next)
     {
       /* block autohide for all windows */
-      if (sensitive)
-        panel_window_thaw_autohide (PANEL_WINDOW (li->data));
-      else
+      if (blocked)
         panel_window_freeze_autohide (PANEL_WINDOW (li->data));
+      else
+        panel_window_thaw_autohide (PANEL_WINDOW (li->data));
     }
 }
 
